unit UnitComport;

interface
uses Windows, Sysutils, UnitErrorCode;

function ComPortConnected(hcom : THandle) : Boolean;
function OpenComPort(var AHandle : THandle; ComNr : Integer) : DWORD;
Function CloseComPort(var hcom : THandle) : DWORD;
function SetTimeOuts(hcom : THandle; wtout, rtout : Cardinal) : DWORD;
function SetTxRxBuffer(hcom : THandle; RxSize, TxSize : Cardinal) : DWORD;
function SetBaud(hcom : THandle; baud : DWORD) : DWORD;
function SendBuf(hcom : THandle; ABuf : Array of Byte; ASize : DWORD) : DWORD;
function SendTxt(hcom : THandle; stxt : String) : DWORD;
function SendTxtCR(hcom : THandle; stxt : String) : DWORD;
function SendTxtCRLF(hcom : THandle; stxt : String) : DWORD;
function ReadTxt(hcom : THandle; var stxt : String) : DWORD;
function ReadTxtCR(hcom : THandle; var stxt : String) : DWORD;


implementation

function ComPortConnected(hcom : THandle) : Boolean;
begin
  Result := (hcom <> INVALID_HANDLE_VALUE);
end;

function OpenComPort(var AHandle : THandle; ComNr : Integer) : DWORD;
begin
  AHandle := CreateFile(PChar('COM'+IntToStr(ComNr)),
                       GENERIC_READ or GENERIC_WRITE,
                       0,
                       nil,
                       OPEN_EXISTING,
                       FILE_ATTRIBUTE_NORMAL,// OR FILE_FLAG_OVERLAPPED,
                       0);
  if AHandle <> INVALID_HANDLE_VALUE then
  begin
    Result := RET_OK;
    SetTimeOuts(Result,1000,1000);
    SetTxRxBuffer(Result,1024,1024);
    SetBaud(Result,19200);
  end else Result := GetLastError;
end;

Function CloseComPort(var hcom : THandle) : DWORD;
begin
  if CloseHandle(hcom) then
  begin
    hcom := INVALID_HANDLE_VALUE;
    Result := RET_OK;
  end else Result := GetLastError;
end;

function SetTimeOuts(hcom : THandle; wtout, rtout : DWORD) : DWORD;
var CommTimeOuts : TCommTimeOuts;
begin
  Result := RET_OK;

  CommTimeOuts.ReadIntervalTimeout := 0;
  CommTimeOuts.ReadTotalTimeoutMultiplier := 0;
  CommTimeOuts.ReadTotalTimeoutConstant := rtout;
  CommTimeOuts.WriteTotalTimeoutMultiplier := 0;
  CommTimeOuts.WriteTotalTimeoutConstant := wtout;
  if not SetCommTimeOuts(hcom,CommTimeOuts) then
    Result := GetLastError;
end;

function SetTxRxBuffer(hcom : THandle; RxSize, TxSize : Cardinal) : DWORD;
begin
  Result := RET_OK;

  if not SetupComm(hcom,RxSize,TxSize) then
    Result := GetLastError;
end;

function SetBaud(hcom : THandle; baud : DWORD) : DWORD;
var dcb : TDCB;
begin
  Result := RET_OK;

  if not GetCommState(hcom,dcb) then
  begin
    Result := GetLastError;
    Exit;
  end;

  with DCB do
  begin
    Baudrate := baud;
    ByteSize := 8;
    Parity   := 0;
    StopBits := 0;
  end;

  if not SetCommState(hcom,dcb) then
    Result := GetLastError;
end;

function SendBuf(hcom : THandle; ABuf : Array of Byte; ASize : DWORD) : DWORD;
var BytesWritten : DWORD;
begin
  Result := RET_OK;
  if WriteFile(hcom,ABuf,ASize,BytesWritten,nil) then
  begin
    if (ASize <> BytesWritten) then
      Result := ERR_NUM_BYTE_WRITTEN;
  end else Result := GetLastError;
end;

function SendTxt(hcom : THandle; stxt : String) : DWORD;
var BytesWritten, ToWrite : DWORD;
begin
  Result := RET_OK;

  ToWrite := length(stxt);
  if WriteFile(hcom,stxt[1],ToWrite,BytesWritten,nil) then
  begin
    if (ToWrite <> BytesWritten) then
      Result := ERR_NUM_BYTE_WRITTEN;
  end else Result := GetLastError;
end;

function SendTxtCR(hcom : THandle; stxt : String) : DWORD;
begin
  stxt := stxt + #13;
  Result := SendTxt(hcom,stxt);
end;

function SendTxtCRLF(hcom : THandle; stxt : String) : DWORD;
begin
  stxt := stxt + #13 + #10;
  Result := SendTxt(hcom,stxt);
end;

function ReadTxt(hcom : THandle; var stxt : String) : DWORD;
var d : array[1..256] of char;
    BytesRead, i : DWORD;
begin
  Result := RET_OK;
  stxt := '';

  if not ReadFile(hcom,d,SizeOf(d),BytesRead,nil) then
  begin
    Result := GetLastError;
    Exit;
  end;

  for i := 1 to BytesRead do
    stxt := stxt + d[i];
end;

function ReadTxtCR(hcom : THandle; var stxt : String) : DWORD;
var d : char;
    BytesRead : DWORD;
    success : BOOL;
begin
  Result := RET_OK;
  stxt := '';

  repeat
    success := ReadFile(hcom,d,1,BytesRead,nil);
    if success and (BytesRead = 1) then
      stxt := stxt + d;
  until (success = false) or ((success = true) and (BytesRead = 0)) or (d = #13);
  
  if (success = false) then
  begin
    Result := GetLastError;
    stxt := '';
  end else if ((success = true) and (BytesRead = 0)) then
  begin
    Result := ERR_COM_TIMEOUT;
    stxt := '';
  end;
end;


end.
