unit UnitErrorCode;

interface
uses Windows, Sysutils;

Const
        RET_OK                          = $00000000;

        ERROR_CATEGORY_OPERATING_SYSTEM = $00000000;
        ERROR_CATEGORY_BASE             = $20000000;
        ERROR_CATEGORY_COM              = ERROR_CATEGORY_BASE + $00010000;
        ERROR_CATEGORY_SG4              = ERROR_CATEGORY_BASE + $00020000;
        ERROR_CATEGORY_GCODE_CONVERTER  = ERROR_CATEGORY_BASE + $00030000;

        ERR_COM_READ                    = (ERROR_CATEGORY_COM+$00000001);
        ERR_COM_WRITE                   = (ERROR_CATEGORY_COM+$00000002);
        ERR_COM_TIMEOUT                 = (ERROR_CATEGORY_COM+$00000003);
        ERR_NUM_BYTE_WRITTEN            = (ERROR_CATEGORY_COM+$00000004);
        ERR_DATA_INVALID                = (ERROR_CATEGORY_COM+$00000005);

        ERR_FIFO_FULL                   = (ERROR_CATEGORY_SG4+$00000001);
        ERR_PARAM_INVALID               = (ERROR_CATEGORY_SG4+$00000002);
        ERR_I2C_PARAM_INVAILD           = (ERROR_CATEGORY_SG4+$00000003);
        ERR_I2C_SCL_INVALID             = (ERROR_CATEGORY_SG4+$00000004);
        ERR_I2C_INVALID_LENGTH          = (ERROR_CATEGORY_SG4+$00000005);
        ERR_I2C_WRITE                   = (ERROR_CATEGORY_SG4+$00000006);
        ERR_I2C_READ                    = (ERROR_CATEGORY_SG4+$00000007);
        ERR_SE_RUNNING                  = (ERROR_CATEGORY_SG4+$00000008);
        ERR_UNKNOWN_CMD                 = (ERROR_CATEGORY_SG4+$00000009);

        ERR_GCODE_NOT_FOUND             = (ERROR_CATEGORY_GCODE_CONVERTER+$00000001);
        ERR_LINE_EMPTY                  = (ERROR_CATEGORY_GCODE_CONVERTER+$00000002);
        ERR_UNKNOWN_GCODE               = (ERROR_CATEGORY_GCODE_CONVERTER+$00000003);
        ERR_BAD_FORMAT                  = (ERROR_CATEGORY_GCODE_CONVERTER+$00000004);
        ERR_GCODE_NO_VALUE              = (ERROR_CATEGORY_GCODE_CONVERTER+$00000005);
        ERR_PARAM_DUPLICATE             = (ERROR_CATEGORY_GCODE_CONVERTER+$00000006);
        ERR_MULTI_GCODE                 = (ERROR_CATEGORY_GCODE_CONVERTER+$00000007);
        ERR_MISSING_PARAM               = (ERROR_CATEGORY_GCODE_CONVERTER+$00000008);
        ERR_BAD_NUMBER                  = (ERROR_CATEGORY_GCODE_CONVERTER+$00000009);
        ERR_GCODE_NOT_SUPPORTED         = (ERROR_CATEGORY_GCODE_CONVERTER+$0000000A);
        ERR_NIL_GCODELINE_OBJECT        = (ERROR_CATEGORY_GCODE_CONVERTER+$0000000B);
        ERR_GCODELINE_OBJECT_EMPTY      = (ERROR_CATEGORY_GCODE_CONVERTER+$0000000C);
        ERR_GCODELIST_EMPTY             = (ERROR_CATEGORY_GCODE_CONVERTER+$0000000D);

        ERR_ERROR_UNKNOWN               = (ERROR_CATEGORY_BASE+$000000FF);


function _SystemErrorMessage(dwErr : DWORD): string;
function cx_GetErrorString(dwErr : DWORD) : String;

implementation

function _SystemErrorMessage(dwErr : DWORD): string;
var
  P: PChar;
begin
  if FormatMessage(Format_Message_Allocate_Buffer + Format_Message_From_System,
                   nil,
                   dwErr,
                   0, //(SUBLANG_ENGLISH_US shl 10) or LANG_NEUTRAL,
                   @P,
                   0,
                   nil) <> 0 then
  begin
    Result := P;
    LocalFree(Integer(P))
  end else Result := '';
end;

function cx_GetErrorString(dwErr : DWORD) : String;
begin
  if dwErr = 0 then
  begin
    Result := 'SUCCESS' + ' (0x'+IntToHex(dwErr,8)+')';
    exit;
  end;

  if dwErr = $FFFFFFFF then dwErr := GetLastError;

  if not ((dwErr and ERROR_CATEGORY_BASE) = ERROR_CATEGORY_BASE) then
  begin
    Result := _SystemErrorMessage(dwErr) + ' (0x'+IntToHex(dwErr,8)+')';
    Exit;
  end;

  case dwErr of
  ERR_COM_READ          : Result := 'ERR_COM_READ';
  ERR_COM_WRITE         : Result := 'ERR_COM_WRITE';
  ERR_COM_TIMEOUT       : Result := 'ERR_COM_TIMEOUT';
  ERR_NUM_BYTE_WRITTEN  : Result := 'ERR_NUM_BYTE_WRITTEN';
  ERR_DATA_INVALID      : Result := 'ERR_DATA_INVALID';

  ERR_FIFO_FULL         : Result := 'ERR_FIFO_FULL';
  ERR_PARAM_INVALID     : Result := 'ERR_PARAM_INVALID';
  ERR_I2C_PARAM_INVAILD : Result := 'ERR_I2C_PARAM';
  ERR_I2C_SCL_INVALID   : Result := 'ERR_I2C_SCL';
  ERR_I2C_WRITE         : Result := 'ERR_I2C_WRITE';
  ERR_I2C_READ          : Result := 'ERR_I2C_READ';
  ERR_SE_RUNNING        : Result := 'ERR_SE_RUNNING';

  ERR_GCODE_NOT_FOUND        : Result := 'ERR_GCODE_NOT_FOUND';
  ERR_LINE_EMPTY             : Result := 'ERR_LINE_EMPTY';
  ERR_UNKNOWN_GCODE          : Result := 'ERR_UNKNOWN_GCODE';
  ERR_BAD_FORMAT             : Result := 'ERR_BAD_FORMAT';
  ERR_GCODE_NO_VALUE         : Result := 'ERR_GCODE_NO_VALUE';
  ERR_PARAM_DUPLICATE        : Result := 'ERR_PARAM_DUPLICATE';
  ERR_MULTI_GCODE            : Result := 'ERR_MULTI_GCODE';
  ERR_MISSING_PARAM          : Result := 'ERR_MISSING_PARAM';
  ERR_BAD_NUMBER             : Result := 'ERR_BAD_NUMBER';
  ERR_GCODE_NOT_SUPPORTED    : Result := 'ERR_GCODE_NOT_SUPPORTED';
  ERR_NIL_GCODELINE_OBJECT   : Result := 'ERR_NIL_GCODELINE_OBJECT';
  ERR_GCODELINE_OBJECT_EMPTY : Result := 'ERR_GCODELINE_OBJECT_EMPTY';
  ERR_GCODELIST_EMPTY        : Result := 'ERR_GCODELIST_EMPTY';

  ERR_UNKNOWN_CMD       : Result := 'ERR_UNKNOWN_CMD';

  else Result := 'ERR_UNKNOWN';
  end;
  Result := Result + ' (0x'+IntToHex(dwErr,8)+')';
end;

end.
