unit UnitSG4;

interface
uses Windows, Sysutils, syncobjs, UnitComport, UnitErrorCode;

Const   CMD_COLON               = ': ';
        CMD_SET_VMAX            = 's1';
        CMD_SET_VMIN            = 's2';
        CMD_SET_ACCEL           = 's3';
        CMD_SET_DECEL           = 's4';
        CMD_SET_VMAX_FIFO       = 's5';
        CMD_SET_VMIN_FIFO       = 's6';
        CMD_SET_ACCEL_FIFO      = 's7';
        CMD_SET_DECEL_FIFO      = 's8';

        CMD_DELAY               = 's40';

        CMD_LINE_RELATIVE       = 's50';
        CMD_LINE_ABSOLUTE       = 's51';
        CMD_CIRCLE_CW_ABS_INC   = 's52'; //absolute endpoint ; incremental center ; Clockwise
        CMD_CIRCLE_CCW_ABS_INC  = 's53'; //absolute endpoint ; incremental center ; Counter Clockwise
        CMD_CIRCLE_CW_ABS_ABS   = 's54'; //absolute endpoint ; absolute center ; Clockwise
        CMD_CIRCLE_CCW_ABS_ABS  = 's55'; //absolute endpoint ; absolute center ; Counter Clockwise
        CMD_CIRCLE_CW_INC_INC   = 's56'; //incremental endpoint ; incremental center ; Clockwise
        CMD_CIRCLE_CCW_INC_INC  = 's57'; //incremental endpoint ; incremental center ; Counter Clockwise
        CMD_CIRCLE_CW_INC_ABS   = 's58'; //incremental endpoint ; absolute center ; Clockwise
        CMD_CIRCLE_CCW_INC_ABS  = 's59'; //incremental endpoint ; absolute center ; Counter Clockwise

        CMD_RADIUS_ACCURACY     = 's60';
        CMD_ACT_POSITION        = 's61';


        CMD_HARDSTOP            = 'd';
        CMD_SOFTSTOP            = 't';
        CMD_FIFO_RESET          = 'r';
        CMD_CONTINUE            = 'c';
        CMD_RESET_INTFLAG       = 'f';

        CMD_GET_VERSION         = 'g0';
        CMD_GET_VMAX            = 'g1';
        CMD_GET_VMIN            = 'g2';
        CMD_GET_VACT            = 'g3';
        CMD_GET_ACCEL           = 'g4';
        CMD_GET_DECEL           = 'g5';
        CMD_GET_POSITION        = 'g6';
        CMD_GET_TARGET_POSITION = 'g7';
        CMD_GET_STATUS          = 'g8';
        CMD_GET_ACT_CMD_NUM     = 'g9';
        CMD_GET_ACTUALS         = 'g10';
        CMD_GET_RADIUS_ERR      = 'g11';

        CMD_I2C_WRITE           = 'iw';
        CMD_I2C_READ            = 'ir';
        CMD_I2C_FREQ            = 'if';

        ST_STOPPED              = 0;
        ST_ACCEL                = 1;
        ST_RUN                  = 2;
        ST_DECEL                = 3;
        ST_RUN_TO_STOP          = 4;
        ST_SOFT_STOP            = 5;
        ST_HARD_STOP            = 6;
        ST_STOPPING             = 7;

function cx_get_err(serr : String) : DWORD;
function cx_check_cmd(scmd, sval : string) : DWORD;

function cx_set_cmd(hcom : THandle; sCmd : String) : DWORD;
function cx_hardstop(hcom : THandle) : DWORD;
function cx_softstop(hcom : THandle) : DWORD;
function cx_fiforeset(hcom : THandle) : DWORD;
function cx_continue(hcom : THandle) : DWORD;
function cx_reset_intflag(hcom : THandle) : DWORD;

function cx_set_params(hcom : THandle; sCmd : String; dwParam : DWORD) : DWORD;
function cx_set_vmax(hcom : THandle; AVmax : DWORD; AFifo : Boolean) : DWORD;
function cx_set_vmin(hcom : THandle; AVmin : DWORD; AFifo : Boolean) : DWORD;
function cx_set_accel(hcom : THandle; AAccel : DWORD; AFifo : Boolean) : DWORD;
function cx_set_decel(hcom : THandle; ADecel : DWORD; AFifo : Boolean) : DWORD;
function cx_set_radius_accuracy(hcom : THandle; AAccuracy : Byte) : DWORD;

function cx_delay(hcom : THandle; ATime : DWORD) : DWORD;
function cx_move_it(hcom : THandle; AMoveCode : String): DWORD;
function cx_set_act_position(hcom : THandle; APosition : String): DWORD;
function cx_set_position(hcom : THandle; APosition : String; ARel : Boolean): DWORD;
function cx_set_circle(hcom : THandle; ACmdNum : DWORD;
                       Ax1, Ax2, AI, AJ : Integer;
                       APlane : Byte; ACW, AIJINC, AEPINC : Boolean): DWORD;

function cx_get_params(hcom : THandle; sCmd : String; var dwParam : DWORD) : DWORD;
function cx_get_paramstr(hcom : THandle; sCmd, sdata : String; var AParam : String) : DWORD;
function cx_get_version(hcom : THandle; var AVersion : DWORD) : DWORD;
function cx_version_to_str(AVersion : DWORD) : String;
function cx_get_vmax(hcom : THandle; var AVmax : DWORD) : DWORD;
function cx_get_vmin(hcom : THandle; var AVmin : DWORD) : DWORD;
function cx_get_vact(hcom : THandle; var AVact : DWORD) : DWORD;
function cx_get_accel(hcom : THandle; var AAccel : DWORD) : DWORD;
function cx_get_decel(hcom : THandle; var ADecel : DWORD) : DWORD;
function cx_get_radius_accuracy(hcom : THandle; var AAccuracy : Byte) : DWORD;
function cx_get_status(hcom : THandle; var AStatus : DWORD) : DWORD;
function cx_get_act_cmdnum(hcom : THandle; var AActNum : DWORD) : DWORD;

function cx_get_position(hcom : THandle; sCmd : String; var Ax, Ay, Az, Au : Integer) : DWORD;
function cx_get_actpos(hcom : THandle; var Ax, Ay, Az, Au : Integer) : DWORD;
function cx_get_targpos(hcom : THandle; var Ax, Ay, Az, Au : Integer) : DWORD;

function cx_get_acts(hcom : THandle; var AERR, AFIFO, ADELAY, ARUN, ASW, AFLAG : Byte;
                                     var AX, AY, AZ, AU : Integer;
                                     var ANUM, AVACT : DWORD) : DWORD;

function cx_writei2c(hcom : THandle; ASlv : Byte; AData : String) : DWORD;
function cx_readi2c(hcom : THandle; ASlv : Byte; Alen : Byte; var AData : String) : DWORD;
function cx_i2cfreq(hcom : THandle; AClk : Byte) : DWORD;

implementation
var CS : TCriticalSection;

function cx_get_err(serr : String) : DWORD;
begin
  Result := ERR_ERROR_UNKNOWN;
  if serr = 'fifo_full' then result := ERR_FIFO_FULL;
  if serr = 'unknown_cmd' then result := ERR_UNKNOWN_CMD;
  if serr = 'param_error' then result := ERR_PARAM_INVALID;
  if serr = 'i2c_param_err' then result := ERR_I2C_PARAM_INVAILD;
  if serr = 'i2c_scl_err' then result := ERR_I2C_SCL_INVALID;
  if serr = 'running' then result := ERR_SE_RUNNING;
end;

function cx_set_cmd(hcom : THandle; sCmd : String) : DWORD;
var stxt : String;
begin
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,sCmd);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,stxt);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(stxt));
    end;
  finally
    CS.Release;
  end;
end;

function cx_hardstop(hcom : THandle) : DWORD;
begin
  Result := cx_set_cmd(hcom,CMD_HARDSTOP);
end;

function cx_softstop(hcom : THandle) : DWORD;
begin
  Result := cx_set_cmd(hcom,CMD_SOFTSTOP);
end;

function cx_fiforeset(hcom : THandle) : DWORD;
begin
  Result := cx_set_cmd(hcom,CMD_FIFO_RESET);
end;

function cx_continue(hcom : THandle) : DWORD;
begin
  Result := cx_set_cmd(hcom,CMD_CONTINUE);
end;

function cx_reset_intflag(hcom : THandle) : DWORD;
begin
  Result := cx_set_cmd(hcom,CMD_RESET_INTFLAG);
end;

function cx_set_params(hcom : THandle; sCmd : String; dwParam : DWORD) : DWORD;
var stxt : String;
begin
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,sCmd + CMD_COLON + IntToStr(dwParam));
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,stxt);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(stxt));
    end;
  finally
    CS.Release;
  end;
end;

function cx_set_vmax(hcom : THandle; AVmax : DWORD; AFifo : Boolean) : DWORD;
var s : String;
begin
  if AFifo then s := CMD_SET_VMAX_FIFO
  else s := CMD_SET_VMAX;
  Result := cx_set_params(hcom,s,AVmax);
end;

function cx_set_vmin(hcom : THandle; AVmin : DWORD; AFifo : Boolean) : DWORD;
var s : String;
begin
  if AFifo then s := CMD_SET_VMIN_FIFO
  else s := CMD_SET_VMIN;
  Result := cx_set_params(hcom,s,AVmin);
end;

function cx_set_accel(hcom : THandle; AAccel : DWORD; AFifo : Boolean) : DWORD;
var s : String;
begin
  if AFifo then s := CMD_SET_ACCEL_FIFO
  else s := CMD_SET_ACCEL;
  Result := cx_set_params(hcom,s,AAccel);
end;

function cx_set_decel(hcom : THandle; ADecel : DWORD; AFifo : Boolean) : DWORD;
var s : String;
begin
  if AFifo then s := CMD_SET_DECEL_FIFO
  else s := CMD_SET_DECEL;
  Result := cx_set_params(hcom,s,ADecel);
end;

function cx_set_radius_accuracy(hcom : THandle; AAccuracy : Byte) : DWORD;
begin
  Result := cx_set_params(hcom,CMD_RADIUS_ACCURACY,AAccuracy);
end;

function cx_delay(hcom : THandle; ATime : DWORD) : DWORD;
begin
  Result := cx_set_params(hcom,CMD_DELAY,ATIME);
end;

function cx_check_cmd(scmd, sval : string) : DWORD;
var colon_pos, semicolon_pos : Integer;
    tmpcmd, sparam : string;
begin
  Result := RET_OK;
  if sval <> '' then
  begin
    colon_pos := pos(':',sval);
    semicolon_pos := pos(';',sval);
    if (colon_pos > 0) and (semicolon_pos > 0) then
    begin
      tmpcmd := copy(sval,1,colon_pos-1);
      if tmpcmd = sCmd then exit
      else if tmpcmd = '?' then
      begin
        sparam := copy(sval,colon_pos+1,semicolon_pos-colon_pos-1);
        Result := cx_get_err(sparam);
      end;
    end else Result := ERR_DATA_INVALID;
  end else Result := ERR_COM_READ;
end;

function cx_move_it(hcom : THandle; AMoveCode : String): DWORD;
var stxt, sCmd : String;
begin
  sCmd := copy(AMoveCode,1,Pos(':',AMoveCode)-1);
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,AMoveCode);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,stxt);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(stxt));
    end;
  finally
    CS.Release;
  end;
end;

function cx_set_act_position(hcom : THandle; APosition : String): DWORD;
var sCmd, stxt : String;
begin
  sCMD := CMD_ACT_POSITION;
  stxt := sCmd + CMD_COLON + APosition;
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,stxt);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,stxt);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(stxt));
    end;
  finally
    CS.Release;
  end;
end;

function cx_set_position(hcom : THandle; APosition : String; ARel : Boolean): DWORD;
var sCmd, stxt : String;
begin
  if ARel then sCmd := CMD_LINE_RELATIVE
  else sCMD := CMD_LINE_ABSOLUTE;
  stxt := sCmd + CMD_COLON + APosition;

  try
    CS.Acquire;
    Result := SendTxtCR(hcom,stxt);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,stxt);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(stxt));
    end;
  finally
    CS.Release;
  end;
end;

function cx_set_circle(hcom : THandle; ACmdNum : DWORD; Ax1, Ax2, AI, AJ : Integer;
                       APlane : Byte; ACW, AIJINC, AEPINC : Boolean): DWORD;
var sCmd, s : String;
begin
  Result := RET_OK;

  if ACW then
  begin
    if AEPINC then
    begin
      if AIJINC then sCmd := CMD_CIRCLE_CW_INC_INC
      else sCmd := CMD_CIRCLE_CW_INC_ABS;
    end else
    begin
      if AIJINC then sCmd := CMD_CIRCLE_CW_ABS_INC
      else sCmd := CMD_CIRCLE_CW_ABS_ABS;
    end;
  end else
  begin
    if AEPINC then
    begin
      if AIJINC then sCmd := CMD_CIRCLE_CCW_INC_INC
      else sCmd := CMD_CIRCLE_CCW_INC_ABS;
    end else
    begin
      if AIJINC then sCmd := CMD_CIRCLE_CCW_ABS_INC
      else sCmd := CMD_CIRCLE_CCW_ABS_ABS;
    end;
  end;
  s := ' n' + IntToStr(ACmdNum);
  case APlane of
  0 : begin
        s := s + ' x' + IntToStr(Ax1);
        s := s + ' y' + IntToStr(Ax2);
      end;
  1 : begin
        s := s + ' x' + IntToStr(Ax1);
        s := s + ' z' + IntToStr(Ax2);
      end;
  2 : begin
        s := s + ' x' + IntToStr(Ax1);
        s := s + ' u' + IntToStr(Ax2);
      end;
  3 : begin
        s := s + ' y' + IntToStr(Ax1);
        s := s + ' z' + IntToStr(Ax2);
      end;
  4 : begin
        s := s + ' y' + IntToStr(Ax1);
        s := s + ' u' + IntToStr(Ax2);
      end;
  5 : begin
        s := s + ' z' + IntToStr(Ax1);
        s := s + ' u' + IntToStr(Ax2);
      end;
  else s := '';
  end;
  if s = '' then exit;
  s := sCmd + s + ' i' + IntToStr(AI);
  s := s + ' j' + IntToStr(AJ);

  try
    CS.Acquire;
    Result := SendTxtCR(hcom,s);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,s);
      if Result = RET_OK then
        Result := cx_check_cmd(sCmd,Trim(s));
    end;
  finally
    CS.Release;
  end;
end;


function cx_get_params(hcom : THandle; sCmd : String; var dwParam : DWORD) : DWORD;
var s, sCmnd, sParam : String;
    COLON_POS, SEMICOLON_POS : Integer;
    Code : Integer;
begin
  Code := 0;
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,sCMD);
    if Result = RET_OK then
    begin
      repeat
        Result := ReadTxtCR(hcom,s);
        s := trim(s);
        if (Result = RET_OK) and (s <> '') then
        begin
          COLON_POS := pos(':',s);
          SEMICOLON_POS := pos(';',s);
          sCmnd := Trim(copy(s,1,COLON_POS-1));
          if sCmnd = sCmd then
          begin
            sParam := Trim(Copy(s,COLON_POS+1,SEMICOLON_POS-COLON_POS-1));
            if (sCmnd = CMD_GET_STATUS) OR (sCmnd = CMD_GET_VERSION) then
              Val('$'+sParam,dwParam,Code)
            else Val(sParam,dwParam,Code);
            if Code <> 0 then Result := ERR_DATA_INVALID;
            break;
          end;
        end;
      until (s = '') OR (Result <> RET_OK);
    end;
  finally
    CS.Release;
  end;
end;

function cx_get_paramstr(hcom : THandle; sCmd, sdata : String; var AParam : String) : DWORD;
var s : String;
    COLON_POS : Integer;
begin
  AParam := '';
  try
    CS.Acquire;
    Result := SendTxtCR(hcom,sCMD + sdata);
    if Result = RET_OK then
    begin
      Result := ReadTxtCR(hcom,s);
      s := trim(s);
      if (Result = RET_OK) then
      begin
        if s <> '' then
        begin
          Result := cx_check_cmd(sCMD,s);
          if Result = RET_OK then
          begin
            COLON_POS := pos(':',s);
            AParam := Trim(Copy(s,COLON_POS+1,Length(s)-COLON_POS));
          end;
        end;
      end else Result := ERR_DATA_INVALID;
    end;
  finally
    CS.Release;
  end;
end;

function cx_get_version(hcom : THandle; var AVersion : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_VERSION,AVersion);
end;

function cx_version_to_str(AVersion : DWORD) : String;
var Lo, hi : byte;
begin
  lo := Byte(AVersion);
  hi := Byte(AVersion shr 8);
  Result := Format('V%u.%2.2u',[hi,lo]);
end;

function cx_get_vmax(hcom : THandle; var AVmax : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_VMAX,AVmax);
end;

function cx_get_vmin(hcom : THandle; var AVmin : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_VMIN,AVmin);
end;

function cx_get_vact(hcom : THandle; var AVact : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_VACT,AVact);
end;

function cx_get_accel(hcom : THandle; var AAccel : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_ACCEL,AAccel);
end;

function cx_get_decel(hcom : THandle; var ADecel : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_DECEL,ADecel);
end;

function cx_get_radius_accuracy(hcom : THandle; var AAccuracy : Byte) : DWORD;
var dwtmp : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_RADIUS_ERR,dwtmp);
  AAccuracy := Byte(dwtmp);
end;

function cx_get_status(hcom : THandle; var AStatus : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_STATUS,AStatus);
end;

function cx_get_act_cmdnum(hcom : THandle; var AActNum : DWORD) : DWORD;
begin
  Result := cx_get_params(hcom,CMD_GET_ACT_CMD_NUM,AActNum);
end;

function cx_get_position(hcom : THandle; sCmd : String; var Ax, Ay, Az, Au : Integer) : DWORD;
var s : String;
    semicolon_pos : Integer;
begin
  Result := cx_get_paramstr(hcom,sCmd,'',s);
  if (Result <> RET_OK) then exit;

  Result := ERR_DATA_INVALID;
  if (s = '') then Exit;

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  Ax := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  Ay := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  Az := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  Au := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  Result := RET_OK;
end;

function cx_get_actpos(hcom : THandle; var Ax, Ay, Az, Au : Integer) : DWORD;
begin
  Result := cx_get_position(hcom,CMD_GET_POSITION,Ax,Ay,Az,Au);
end;

function cx_get_targpos(hcom : THandle; var Ax, Ay, Az, Au : Integer) : DWORD;
begin
  Result := cx_get_position(hcom,CMD_GET_TARGET_POSITION,Ax,Ay,Az,Au);
end;

function cx_get_acts(hcom : THandle; var AERR, AFIFO, ADELAY, ARUN, ASW, AFLAG : Byte;
                                     var AX, AY, AZ, AU : Integer;
                                     var ANUM, AVACT : DWORD) : DWORD;
var s : String;
    semicolon_pos : Integer;
    dwparam : dword;
begin
  Result := cx_get_paramstr(hcom,CMD_GET_ACTUALS,'',s);
  if Result <> RET_OK then exit;

  Result := ERR_DATA_INVALID;
  if (s = '') then Exit;

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  dwparam := StrToInt('$'+Trim(copy(s,1,semicolon_pos-1)));
  ASW := (dwparam and $000000FF);
  ARUN := (dwparam and $00000F00) shr 8;
  AFIFO := (dwparam and $00001000) shr 12;
  AERR := (dwparam and $00002000) shr 13;
  ADELAY := (dwparam and $00004000) shr 14;
  AFLAG := (dwparam and $FF000000) shr 24;
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  ANUM := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  AX := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  AY := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  AZ := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  AU := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  semicolon_pos := pos(';',s);
  if (semicolon_pos = 0) then exit;
  AVACT := StrToInt(Trim(copy(s,1,semicolon_pos-1)));
  delete(s,1,semicolon_pos);

  Result := RET_OK;
end;

function cx_writei2c(hcom : THandle; ASlv : Byte; AData : String) : DWORD;
var len : Integer;
    sCmd, sdata, sparam : String;
begin
  sCmd := CMD_I2C_WRITE;

  ASlv := ASlv and $FE;

  len := length(AData) DIV 2;
  if (len = 0) OR (len > 32) then
  begin
    Result := ERR_I2C_INVALID_LENGTH;
    Exit;
  end;
  
  sdata := CMD_COLON + IntToHex(ASlv,2) + IntToHex(len,2) + Copy(AData,1,len*2);

  Result := cx_get_paramstr(hcom,sCmd,sdata,sparam);
  if Result <> RET_OK then exit;
  
  if StrToInt('$'+copy(sparam,1,2)) <> len then
    Result := ERR_I2C_WRITE;
end;

function cx_readi2c(hcom : THandle; ASlv : Byte; Alen : Byte; var AData : String) : DWORD;
var sCmd, sdata, sparam : String;
begin
  sdata := '';
  sCmd := CMD_I2C_Read;

  ASlv := (ASlv and $FE) OR $01;

  if (Alen = 0) OR (Alen > 32) then
  begin
    Result := ERR_I2C_INVALID_LENGTH;
    Exit;
  end;

  sdata := CMD_COLON + IntToHex(ASlv,2) + IntToHex(Alen,2);
  Result := cx_get_paramstr(hcom,sCmd,sdata,sparam);
  if Result <> RET_OK then exit;

  if (StrToInt('$'+copy(sparam,1,2)) <> Alen) then
  begin
    Result := ERR_I2C_READ;
    Exit;
  end;

  Adata := Copy(sparam,3,Length(sparam)-3);
  if ((Length(Adata) div 2) <> Alen) then
  begin
    Adata := '';
    Result := ERR_I2C_READ;
    Exit;
  end;
end;

function cx_i2cfreq(hcom : THandle; AClk : Byte) : DWORD;
var sCmd : String;
begin
  sCmd := CMD_I2C_FREQ;

  if (AClk <> 25) and (AClk <> 50) and (AClk <> 100) then
  begin
    Result := ERR_I2C_SCL_INVALID;
    Exit;
  end;
  Result := cx_set_params(hcom,sCmd,AClk);
end;

initialization
  CS := TCriticalSection.Create;

finalization
  CS.Free;

end.
