unit uGCodeAnatomizer;

interface
uses Windows, Sysutils, classes, udef, UnitErrorCode;

Type
        TCharSet = Set of Char;

        TStringObj = class(TObject)
        private
          fStr: String;
        public
          constructor Create(const AStr: String) ;
          property Str: String read FStr write FStr;
        end;

        TGCodeLine = Class
        Private
          IDCharSet : set of Char;
          ValueCharSet : set of Char;
          FGCode : String;
          FDecimal : Char;
          procedure SetDecimal(const Value : Char);
          function  ReadDecimal : Char;
          function ReadGCode: String;
          procedure clear_paramlist;
          function replace_separator(sval : String) : String;
          procedure remove_whitespace;
          function get_value(CodeChar: Char; var svalue : String; DeleteAfterRead : Boolean = false) : DWORD;
          function remove_comments : Boolean;
        Public
          GCodeList : TStringList;
          MCodeList : TStringList;
          ParamList : TStringList;
          MsgList   : TStringList;
          Property DecimalPoint : Char read ReadDecimal write SetDecimal;
          Property GCode : String read ReadGCode;
          constructor Create;
          destructor Destroy; override;
          function anatomize(AGCode : String; var ADataAvailable : Boolean) : DWORD;
          function get_param_int(sParam : String; var value : integer; FreeObj : Boolean = false) : integer;
          function get_param_dword(sParam : String; var value : dword; FreeObj : Boolean = false) : integer;
          function get_param_double(sParam : String; var value : double; FreeObj : Boolean = false) : integer;
          function get_param_string(sParam : String; var value : string; FreeObj : Boolean = false) : integer;
          function IndexOfInt(AList : TStringList; AValue : Integer) : Integer;
          function anatomized : boolean;
        end;


implementation

{ TStringObj }

constructor TStringObj.Create(const AStr: String);
begin
  inherited Create;
  FStr := AStr;
end;

{ TGCodeLine }

constructor TGCodeLine.Create;
begin
  inherited Create;
  IDCharSet := ['G','M','X','Y','Z','I','J','K','R','Q','L','F','S','T','D','H','N','O','P'];
  ValueCharSet := ['0','1','2','3','4','5','6','7','8','9','.','-','+'];
  GCodeList := TStringList.Create;
  MCodeList := TStringList.Create;
  ParamList := TStringList.Create;
  MsgList   := TStringList.Create;
  FDecimal  := DecimalSeparator;
  FGCode    := '';
end;

destructor TGCodeLine.Destroy;
begin
  GCodeList.Free;
  MCodeList.Free;
  clear_paramlist;
  ParamList.Free;
  MsgList.Free;
  inherited Destroy;
end;

procedure TGCodeLine.clear_paramlist;
var i : integer;
begin
  if ParamList.Count > 0 then
  begin
    for i := 0 to ParamList.Count - 1 do
    begin
      if ParamList.Objects[i] <> nil then
      begin
        ParamList.Objects[i].Free;
        ParamList.Objects[i] := nil;
      end;
    end;
  end;
  ParamList.Clear;
end;

function TGCodeLine.ReadDecimal: Char;
begin
  Result := FDecimal;
end;

function TGCodeLine.ReadGCode: String;
begin
  Result := FGCode;
end;

procedure TGCodeLine.SetDecimal(const Value : Char);
begin
  FDecimal := Value;
end;

function TGCodeLine.replace_separator(sval : String) : String;
var decimal_to_replace : Char;
begin
  if FDecimal = '.' then decimal_to_replace := ','
  else decimal_to_replace := '.';

  Result := StringReplace(sval,decimal_to_replace,FDecimal,[rfReplaceAll]);
end;

procedure TGCodeLine.remove_whitespace;
begin
  FGCode := StringReplace(FGCode,' ','',[rfReplaceAll, rfIgnoreCase]);
end;

function TGCodeLine.get_value(CodeChar: Char; var svalue : String;
                              DeleteAfterRead : Boolean = false) : DWORD;
var loop : Integer;
    CodePos : Integer;
    ReadFromPos, ReadToPos : Integer;
    delete_from, delete_to : Integer;
    plus_counter, minus_counter, point_counter : integer;
begin
  Result := RET_OK;
  svalue := '';
  plus_counter := 0;
  minus_counter := 0;
  point_counter := 0;

  if FGCode = '' then
  begin
    Result := ERR_LINE_EMPTY;
    Exit;
  end;

  if not (CodeChar in IDCharSet) then
  begin
    Result := ERR_UNKNOWN_GCODE;
    Exit;
  end;

  CodePos := Pos(CodeChar,FGCode);
  if CodePos = 0 then
  begin
    Result := ERR_GCODE_NOT_FOUND;
    Exit;
  end;

  ReadFromPos := CodePos + 1;
  ReadToPos   := length(FGCode);

  delete_from := ReadFromPos - 1;
  delete_to   := 0;

  for loop := ReadFromPos to ReadToPos do
  begin
    if (FGCode[loop] in IDCharSet) then
      break;

    if (FGCode[loop] in ValueCharSet) then
    begin
      if FGCode[loop] = '+' then
      begin
        inc(plus_counter);
        if loop > ReadFromPos then
        begin
          Result := ERR_BAD_FORMAT;
          Exit;
        end;
      end;

      if FGCode[loop] = '-' then
      begin
        inc(minus_counter);
        if loop > ReadFromPos then
        begin
          Result := ERR_BAD_FORMAT;
          Exit;
        end;
      end;

      if FGCode[loop] = '.' then
      begin
        inc(point_counter);
        if loop = ReadFromPos then svalue := '0';
      end;

      if (plus_counter > 1) or (minus_counter > 1) or (point_counter > 1) then
      begin
        Result := ERR_BAD_FORMAT;
        Exit;
      end;

      svalue := svalue + FGCode[loop];
      inc(delete_to);
    end else break;

  end;

  if svalue = '' then
  begin
    Result := ERR_GCODE_NO_VALUE;
    Exit;
  end;

  if DeleteAfterRead and (delete_to > 0) then
    delete(FGCode,delete_from,delete_to + 1);
end;

function TGCodeLine.remove_comments: boolean;
var left_parenthesis_pos, right_parenthesis_pos, semicolon_pos : Integer;
begin
  Result := true;

  semicolon_pos := Pos(';',FGCode);
  left_parenthesis_pos := Pos('(',FGCode);
  right_parenthesis_pos := Pos(')',FGCode);

  if semicolon_pos > 0 then
  begin
    if (left_parenthesis_pos = 0) OR (semicolon_pos < left_parenthesis_pos) then
    begin
      delete(FGCode,semicolon_pos,length(FGCode)-semicolon_pos+1);
      Exit;
    end;
  end;
  if (left_parenthesis_pos = 0) and (right_parenthesis_pos = 0) then
    Exit;

  if (left_parenthesis_pos = 0) and (right_parenthesis_pos > 0) then
  begin
    Result := false;
    Exit;
  end;

  while (left_parenthesis_pos > 0) and (right_parenthesis_pos > 0) do
  begin
    if (right_parenthesis_pos > left_parenthesis_pos) then
    begin
      Delete(FGCode,left_parenthesis_pos,right_parenthesis_pos - left_parenthesis_pos + 1);
      Result := true;
    end else
    begin
      Result := false;
      exit;
    end;
    left_parenthesis_pos := Pos('(',FGCode);
    right_parenthesis_pos := Pos(')',FGCode);
  end;

  if (left_parenthesis_pos > 0) and (right_parenthesis_pos = 0) then
    Result := false;
end;

function TGCodeLine.anatomize(AGCode: String; var ADataAvailable : Boolean) : DWORD;
var i : integer;
    svalue : string;
    dwerr : DWORD;
    param_counter : integer;
begin
  Result := RET_OK;
  ADataAvailable := false;

  GCodeList.Clear;
  MCodeList.Clear;
  clear_paramlist;
  MsgList.Clear;

  FGCode := Trim(AGCode);
  if not remove_comments then
  begin
   Result := ERR_BAD_FORMAT;
   Exit;
  end;

  remove_whitespace;
  if FGCode = '' then
  begin
    Result := ERR_LINE_EMPTY;
    Exit;
  end;

  FGCode := UpperCase(FGCode);

  repeat
    dwerr := get_value('G',svalue,DELETE_GCODE);
    case dwerr of
    RET_OK : begin
               if Pos('.',svalue) > 0 then Delete(svalue,Pos('.',svalue),1)
               else svalue := svalue + '0';
               GCodeList.Add(svalue);
             end;
    ERR_GCODE_NOT_FOUND,
    ERR_LINE_EMPTY:     break; // go to next

    ERR_UNKNOWN_GCODE,
    ERR_BAD_FORMAT,
    ERR_GCODE_NO_VALUE: begin
                          Result := dwerr;
                          Exit;
                        end;
    end;
  until (dwerr = ERR_GCODE_NOT_FOUND);

  repeat
    dwerr := get_value('M',svalue,DELETE_GCODE);
    case dwerr of
    RET_OK : begin
               if Pos('.',svalue) > 0 then Delete(svalue,Pos('.',svalue),1)
               else svalue := svalue + '0';
               MCodeList.Add(svalue);
             end;
    ERR_GCODE_NOT_FOUND,
    ERR_LINE_EMPTY:     break;

    ERR_UNKNOWN_GCODE,
    ERR_BAD_FORMAT,
    ERR_GCODE_NO_VALUE: begin
                          Result := dwerr;
                          Exit;
                        end;
    end;
  until (dwerr = ERR_GCODE_NOT_FOUND);

  for i := 0 to Length(cParams) - 1 do
  begin
    param_counter := 0;
    repeat
      dwerr := get_value(cParams[i],svalue,DELETE_GCODE);
      case dwerr of
      RET_OK : begin
                 inc(param_counter);
                 if param_counter > 1 then
                 begin
                   Result := ERR_PARAM_DUPLICATE;
                   Exit;
                 end;
                 svalue := replace_separator(svalue);
                 ParamList.AddObject(cParams[i],TStringObj.Create(svalue));
               end;
      ERR_GCODE_NOT_FOUND,
      ERR_LINE_EMPTY:     break;

      ERR_UNKNOWN_GCODE,
      ERR_BAD_FORMAT,
      ERR_GCODE_NO_VALUE: begin
                            Result := dwerr;
                            Exit;
                          end;
      end;
    until (dwerr = ERR_GCODE_NOT_FOUND);
  end;


  ADataAvailable := (GCodeList.Count > 0) OR (MCodeList.Count > 0) OR (ParamList.Count > 0);
  if not ADataAvailable then Result := ERR_GCODELIST_EMPTY;
end;

function TGCodeLine.anatomized: boolean;
begin
  Result := (GCodeList.Count > 0) OR (MCodeList.Count > 0) OR (ParamList.Count > 0);
end;

function TGCodeLine.get_param_int(sParam : String; var value: integer;
                                  FreeObj : Boolean): integer;
var idx : integer;
    code : integer;
begin
  Result := -1;
  idx := ParamList.IndexOf(sParam);
  if idx >= 0 then
  begin
    if ParamList.Objects[idx] = nil then exit;
    val(TStringObj(ParamList.Objects[idx]).Str,value, code);
    if code <> 0 then result := 1
    else result := 0;
    if FreeObj then
    begin
      ParamList.Objects[idx].Free;
      ParamList.Objects[idx] := nil;
    end;
  end;
end;

function TGCodeLine.get_param_dword(sParam: String; var value: dword;
                                    FreeObj: Boolean): integer;
var idx : integer;
    code : integer;
begin
  Result := -1;
  idx := ParamList.IndexOf(sParam);
  if idx >= 0 then
  begin
    if ParamList.Objects[idx] = nil then exit;
    val(TStringObj(ParamList.Objects[idx]).Str,value, code);
    if code <> 0 then result := 1
    else result := 0;
    if FreeObj then
    begin
      ParamList.Objects[idx].Free;
      ParamList.Objects[idx] := nil;
    end;
  end;
end;

function TGCodeLine.get_param_double(sParam: String; var value: double;
                                     FreeObj: Boolean): integer;
var idx : integer;
begin
  Result := -1;
  idx := ParamList.IndexOf(sParam);
  if idx >= 0 then
  begin
    if ParamList.Objects[idx] = nil then exit;
    try
      value := StrToFloat(TStringObj(ParamList.Objects[idx]).Str);
      result := 0;
    except
      result := 1;
    end;
    if FreeObj then
    begin
      ParamList.Objects[idx].Free;
      ParamList.Objects[idx] := nil;
    end;
  end;
end;

function TGCodeLine.get_param_string(sParam: String; var value: string;
                                     FreeObj: Boolean): integer;
var idx : integer;
begin
  Result := -1;
  idx := ParamList.IndexOf(sParam);
  if idx >= 0 then
  begin
    if ParamList.Objects[idx] = nil then exit;
    value := TStringObj(ParamList.Objects[idx]).Str;
    if trim(value) <> '' then result := 0
    else result := 1;
    if FreeObj then
    begin
      ParamList.Objects[idx].Free;
      ParamList.Objects[idx] := nil;
    end;
  end;
end;

function TGCodeLine.IndexOfInt(AList: TStringList; AValue: Integer): Integer;
begin
  for Result := 0 to AList.Count - 1 do
    if (StrToIntDef(AList.Strings[Result],$FFFF) = AValue) then Exit;
  Result := -1;
end;

end.
