unit uGCodeParameter;

interface
uses Windows, Sysutils, Classes, udef, UnitErrorCode;

type    TFixtureOffSet = Record
          X : Double;
          Y : Double;
          Z : Double;
        end;

        TGCodeParam = Class
        private
          FDwellUnit : Boolean; //TRUE = Seconds; False = Milliseconds
          FUnitsPerStep : Double; // resolution of maschine and so on...
          FLengthUnit : Boolean; //TRUE = Inches; False = mm

          FX : Double;
          FXChanged : Boolean;
          FY : Double;
          FYChanged : Boolean;
          FZ : Double;
          FZChanged : Boolean;
          FI : Double;
          FIChanged : Boolean;
          FJ : Double;
          FJChanged : Boolean;
          FK : Double;
          FKChanged : Boolean;
          FF : Double;
          FFChanged : Boolean;
          FD : Double;
          FDChanged : Boolean;
          FH : Double;
          FHChanged : Boolean;
          FL : Double;
          FLChanged : Boolean;
          FN : Double;
          FNChanged : Boolean;
          FP : Double;
          FPChanged : Boolean;
          FQ : Double;
          FQChanged : Boolean;
          FR : Double;
          FRChanged : Boolean;
          FS : Double;
          FSChanged : Boolean;
          FT : Double;
          FTChanged : Boolean;
          FO : Double;
          FOChanged : Boolean;

          FActiveMotion : Integer;
          FActiveMotionChanged : Boolean;
          FActivePlane : Integer;
          FDistanceMode : Integer;
          FArcDistanceMode : Integer;

          FActiveFixture : Byte;
          FFixtureOffSets : array[0..9] of TFixtureOffSet;

          FVmax  : DWORD;
          FVmin  : DWORD;
          FAccel : DWORD;
          FDecel : DWORD;
          FVminPercentOfVmax : Byte; //0= not active; 1..100% of Vmax
          FAccelDecelTime : DWORD; //milliseconds; 0= not active; >0 time to accelerate from Vmin to Vmax

          FVmaxUpperLimit : DWORD;
          FVmaxLowerLimit : DWORD;
          FVminUpperLimit : DWORD;
          FVminLowerLimit : DWORD;


          function ReadLengthUnit: Boolean;
          function ReadUnitsPerStep: Double;
          procedure SetLengthUnit(const Value: Boolean);
          procedure SetUnitsPerStep(const Value: Double);
          function ReadDwellUnit: Boolean;
          procedure SetDwellUnit(const Value: Boolean);

          function Read_X: Double;
          function Read_XSteps: Integer;
          function Read_XChanged: Boolean;
          procedure Set_X(const Value: Double);
          function Read_Y: Double;
          function Read_YSteps: Integer;
          function Read_YChanged: Boolean;
          procedure Set_Y(const Value: Double);
          function Read_Z: Double;
          function Read_ZSteps: Integer;
          function Read_ZChanged: Boolean;
          procedure Set_Z(const Value: Double);
          function Read_I: Double;
          function Read_ISteps: Integer;
          function Read_IChanged: Boolean;
          procedure Set_I(const Value: Double);
          function Read_J: Double;
          function Read_JSteps: Integer;
          function Read_JChanged: Boolean;
          procedure Set_J(const Value: Double);
          function Read_K: Double;
          function Read_KSteps: Integer;
          function Read_KChanged: Boolean;
          procedure Set_K(const Value: Double);
          function Read_F: Double;
          function Read_FChanged: Boolean;
          procedure Set_F(const Value: Double);
          function Read_D: Double;
          function Read_DChanged: Boolean;
          procedure Set_D(const Value: Double);
          function Read_H: Double;
          function Read_HChanged: Boolean;
          procedure Set_H(const Value: Double);
          function Read_L: Double;
          function Read_LChanged: Boolean;
          procedure Set_L(const Value: Double);
          function Read_N: Double;
          function Read_NChanged: Boolean;
          procedure Set_N(const Value: Double);
          function Read_P: Double;
          function Read_PChanged: Boolean;
          procedure Set_P(const Value: Double);
          function Read_Q: Double;
          function Read_QChanged: Boolean;
          procedure Set_Q(const Value: Double);
          function Read_R: Double;
          function Read_RChanged: Boolean;
          procedure Set_R(const Value: Double);
          function Read_S: Double;
          function Read_SChanged: Boolean;
          procedure Set_S(const Value: Double);
          function Read_T: Double;
          function Read_TChanged: Boolean;
          procedure Set_T(const Value: Double);
          function Read_O: Double;
          function Read_OChanged: Boolean;
          procedure Set_O(const Value: Double);

          function ReadActiveMotion: Integer;
          procedure SetActiveMotion(const Value: Integer);
          function ReadActiveMotionChanged: Boolean;
          function ReadActivePlane: Integer;
          procedure SetActivePlane(const Value: Integer);
          function ReadVmaxSteps: DWORD;
          function ReadVmaxUnits: Double;
          procedure SetVmaxSteps(const Value: DWORD);
          procedure SetVmaxUnits(const Value: double);
          function ReadVminSteps: DWORD;
          function ReadVminUnits: Double;
          procedure SetVminSteps(const Value: DWORD);
          procedure SetVminUnits(const Value: Double);
          function ReadAccelSteps: DWORD;
          function ReadAccelUnits: Double;
          procedure SetAccelSteps(const Value: DWORD);
          procedure SetAccelUnits(const Value: Double);
          function ReadDecelSteps: DWORD;
          function ReadDecelUnits: Double;
          procedure SetDecelSteps(const Value: DWORD);
          procedure SetDecelUnits(const Value: Double);
          function ReadVminPercentOfVmax: Byte;
          procedure SetVminPercentOfVmax(const Value: Byte);
          function ReadIsVminPercentOfVmax: Boolean;
          function ReadAccelDecelTime: DWORD;
          procedure SetAccelDecelTime(const Value: DWORD);

          function ReadVmaxUpperLimitSteps: DWORD;
          procedure SetVmaxUpperLimitSteps(const Value: DWORD);
          function ReadVmaxUpperLimitUnits: Double;
          procedure SetVmaxUpperLimitUnits(const Value: Double);
          function ReadVminUpperLimitSteps: DWORD;
          procedure SetVminUpperLimitSteps(const Value: DWORD);
          function ReadVminUpperLimitUnits: Double;
          procedure SetVminUpperLimitUnits(const Value: Double);
          function ReadVmaxLowerLimitSteps: DWORD;
          procedure SetVmaxLowerLimitSteps(const Value: DWORD);
          function ReadVmaxLowerLimitUnits: Double;
          procedure SetVmaxLowerLimitUnits(const Value: Double);
          function ReadVminLowerLimitSteps: DWORD;
          procedure SetVminLowerLimitSteps(const Value: DWORD);
          function ReadVminLowerLimitUnits: Double;
          procedure SetVminLowerLimitUnits(const Value: Double);

          function ReadActiveFixture: Byte;
          procedure WriteActiveFixture(const Value: Byte);
          function ReadFixtureXOffSet: Double;
          procedure WriteFixtureXOffSet(const Value: Double);
          function ReadFixtureXOffSetSteps: Integer;
          procedure WriteFixtureXOffSetSteps(const Value: Integer);
          function ReadFixtureYOffSet: Double;
          procedure WriteFixtureYOffSet(const Value: Double);
          function ReadFixtureYOffSetSteps: Integer;
          procedure WriteFixtureYOffSetSteps(const Value: Integer);
          function ReadFixtureZOffSet: Double;
          procedure WriteFixtureZOffSet(const Value: Double);
          function ReadFixtureZOffSetSteps: Integer;
          procedure WriteFixtureZOffSetSteps(const Value: Integer);
          function ReadDistanceMode: Integer;
          procedure SetDistanceMode(const Value: Integer);
    function ReadArcDistanceMode: Integer;
    procedure SetArcDistanceMode(const Value: Integer);

        protected

        public
          constructor Create;
          destructor Destroy; override;

          Property UnitsPerStep : Double read ReadUnitsPerStep write SetUnitsPerStep;
          Property LengthUnit : Boolean read ReadLengthUnit write SetLengthUnit;
          Property DwellUnit : Boolean read ReadDwellUnit write SetDwellUnit;

          Property _X : Double read Read_X write Set_X;
          Property _XSteps : Integer read Read_XSteps;
          Property _XChanged : Boolean read Read_XChanged;
          Property _Y : Double read Read_Y write Set_Y;
          Property _YSteps : Integer read Read_YSteps;
          Property _YChanged : Boolean read Read_YChanged;
          Property _Z : Double read Read_Z write Set_Z;
          Property _ZSteps : Integer read Read_ZSteps;
          Property _ZChanged : Boolean read Read_ZChanged;
          Property _I : Double read Read_I write Set_I;
          Property _ISteps : Integer read Read_ISteps;
          Property _IChanged : Boolean read Read_IChanged;
          Property _J : Double read Read_J write Set_J;
          Property _JSteps : Integer read Read_JSteps;
          Property _JChanged : Boolean read Read_JChanged;
          Property _K : Double read Read_K write Set_K;
          Property _KSteps : Integer read Read_KSteps;
          Property _KChanged : Boolean read Read_KChanged;
          Property _F : Double read Read_F write Set_F;
          Property _FChanged : Boolean read Read_FChanged;
          Property _D : Double read Read_D write Set_D;
          Property _DChanged : Boolean read Read_DChanged;
          Property _H : Double read Read_H write Set_H;
          Property _HChanged : Boolean read Read_HChanged;
          Property _L : Double read Read_L write Set_L;
          Property _LChanged : Boolean read Read_LChanged;
          Property _N : Double read Read_N write Set_N;
          Property _NChanged : Boolean read Read_NChanged;
          Property _P : Double read Read_P write Set_P;
          Property _PChanged : Boolean read Read_PChanged;
          Property _Q : Double read Read_Q write Set_Q;
          Property _QChanged : Boolean read Read_QChanged;
          Property _R : Double read Read_R write Set_R;
          Property _RChanged : Boolean read Read_RChanged;
          Property _S : Double read Read_S write Set_S;
          Property _SChanged : Boolean read Read_SChanged;
          Property _T : Double read Read_T write Set_T;
          Property _TChanged : Boolean read Read_TChanged;
          Property _O : Double read Read_O write Set_O;
          Property _OChanged : Boolean read Read_OChanged;

          Property ActiveMotion : Integer read ReadActiveMotion write SetActiveMotion;
          Property ActiveMotionChanged : Boolean read ReadActiveMotionChanged;
          Property ActivePlane : Integer read ReadActivePlane write SetActivePlane;
          Property DistanceMode : Integer read ReadDistanceMode write SetDistanceMode;
          Property ArcDistanceMode : Integer read ReadArcDistanceMode write SetArcDistanceMode;

          Property VmaxSteps : DWORD read ReadVmaxSteps write SetVmaxSteps;
          Property VmaxUnits : Double read ReadVmaxUnits write SetVmaxUnits;
          Property VminSteps : DWORD read ReadVminSteps write SetVminSteps;
          Property VminUnits : Double read ReadVminUnits write SetVminUnits;
          Property AccelSteps : DWORD read ReadAccelSteps write SetAccelSteps;
          Property AccelUnits : Double read ReadAccelUnits write SetAccelUnits;
          Property DecelSteps : DWORD read ReadDecelSteps write SetDecelSteps;
          Property DecelUnits : Double read ReadDecelUnits write SetDecelUnits;
          Property VminPercentOfVmax : Byte read ReadVminPercentOfVmax write SetVminPercentOfVmax;
          Property IsVminPercentOfVmax : Boolean read ReadIsVminPercentOfVmax;
          Property AccelDecelTime : DWORD read ReadAccelDecelTime write SetAccelDecelTime;

          Property VmaxUpperLimitSteps : DWORD read ReadVmaxUpperLimitSteps write SetVmaxUpperLimitSteps;
          Property VmaxUpperLimitUnits : Double read ReadVmaxUpperLimitUnits write SetVmaxUpperLimitUnits;
          Property VminUpperLimitSteps : DWORD read ReadVminUpperLimitSteps write SetVminUpperLimitSteps;
          Property VminUpperLimitUnits : Double read ReadVminUpperLimitUnits write SetVminUpperLimitUnits;

          Property VmaxLowerLimitSteps : DWORD read ReadVmaxLowerLimitSteps write SetVmaxLowerLimitSteps;
          Property VmaxLowerLimitUnits : Double read ReadVmaxLowerLimitUnits write SetVmaxLowerLimitUnits;
          Property VminLowerLimitSteps : DWORD read ReadVminLowerLimitSteps write SetVminLowerLimitSteps;
          Property VminLowerLimitUnits : Double read ReadVminLowerLimitUnits write SetVminLowerLimitUnits;

          Property ActiveFixture : Byte read ReadActiveFixture write WriteActiveFixture;
          Property XOffSet : Double read ReadFixtureXOffSet write WriteFixtureXOffSet;
          Property XOffSetSteps : Integer read ReadFixtureXOffSetSteps write WriteFixtureXOffSetSteps;
          Property YOffSet : Double read ReadFixtureYOffSet write WriteFixtureYOffSet;
          Property YOffSetSteps : Integer read ReadFixtureYOffSetSteps write WriteFixtureYOffSetSteps;
          Property ZOffSet : Double read ReadFixtureZOffSet write WriteFixtureZOffSet;
          Property ZOffSetSteps : Integer read ReadFixtureZOffSetSteps write WriteFixtureZOffSetSteps;

          function InchToMillimeter(AInch : Double) : Double;
          function MillimeterToInch(AMillimeter : Double) : Double;

          function LengthToSteps(ALength : Double) : Integer; Overload;
          function LengthToSteps(ALength, AUnitsPerStep : Double) : Integer; Overload;
          function StepsToLength(ASteps : Integer) : Double; Overload;
          function StepsToLength(ASteps : Integer; AUnitsPerStep : Double) : Double; Overload;
          function VelocityToStepsPerSecond(AVelocity : Double) : DWORD; Overload;
          function VelocityToStepsPerSecond(AVelocity, AUnitsPerStep : Double) : DWORD; Overload;
          function StepsPerSecondToVelocity(AStepsPerSec : DWORD) : Double; Overload;
          function StepsPerSecondToVelocity(AStepsPerSec : DWORD; AUnitsPerStep : Double) : Double; Overload;
          function AccelToStepsPerSecSec(AAccel : Double) : DWORD; Overload;
          function AccelToStepsPerSecSec(AAccel, AUnitsPerStep : Double) : DWORD; Overload;
          function StepsPerSecSecToAccel(AStepsPerSecSec : DWORD) : Double; Overload;
          function StepsPerSecSecToAccel(AStepsPerSecSec : DWORD; AUnitsPerStep : Double) : Double; Overload;
          function DwellToSG4Time(fTime : double) : DWORD;
          procedure ResetFlags;
        end;



implementation

{ TGCodeParam }

constructor TGCodeParam.Create;
begin
  inherited Create;

  FLengthUnit   := false;       // Length Unit is [mm]
  FUnitsPerStep := 1;           // 1 mm/step
  FDwellUnit    := true;        // Dwell time in seconds

  FVmaxUpperLimit := 100000;
  FVmaxLowerLimit := 1;
  FVminUpperLimit := 100000;
  FVminLowerLimit := 1;
  
  FVminPercentOfVmax := 40;     // set Vmin to 40% of Vmax
  FAccelDecelTime := 500;       // acceleration from Vmin to Vmax within 500ms
  SetVmaxSteps(1000);           // set Vmax to 1000 steps/s
                                // set Vmin to 400 steps/s


  FX := 0;
  FXChanged := false;
  FY := 0;
  FYChanged := false;
  FZ := 0;
  FZChanged := false;
  FI := 0;
  FIChanged := false;
  FJ := 0;
  FJChanged := false;
  FK := 0;
  FKChanged := false;
  FF := 0;
  FFChanged := false;
  FD := 0;
  FDChanged := false;
  FH := 0;
  FHChanged := false;
  FL := 0;
  FLChanged := false;
  FN := 0;
  FNChanged := false;
  FP := 0;
  FPChanged := false;
  FQ := 0;
  FQChanged := false;
  FR := 0;
  FRChanged := false;
  FS := 0;
  FSChanged := false;
  FT := 0;
  FTChanged := false;
  FO := 0;
  FOChanged := false;

  FActiveMotion := G_80;
  FActiveMotionChanged := false;
  FActivePlane := G_17; //XY-Plane
  ActiveFixture := 0;
  FDistanceMode := G_90; //absolute distance mode
  FArcDistanceMode := G_91_1; //incremental arc distance mode
end;

destructor TGCodeParam.Destroy;
begin

  inherited Destroy;
end;

function TGCodeParam.InchToMillimeter(AInch: Double): Double;
begin
  Result := AInch * INCH_TO_MM;
end;

function TGCodeParam.MillimeterToInch(AMillimeter: Double): Double;
begin
  Result := AMillimeter * MM_TO_INCH;
end;

function TGCodeParam.LengthToSteps(ALength: Double): Integer;
begin
  Result := Round(ALength / FUnitsPerStep);
end;

function TGCodeParam.LengthToSteps(ALength, AUnitsPerStep: Double): Integer;
begin
  Result := Round(ALength / AUnitsPerStep);
end;

function TGCodeParam.StepsToLength(ASteps: Integer): Double;
begin
  Result := ASteps * FUnitsPerStep;
end;

function TGCodeParam.StepsToLength(ASteps: Integer;
                                   AUnitsPerStep: Double): Double;
begin
  Result := ASteps * AUnitsPerStep;
end;

function TGCodeParam.StepsPerSecondToVelocity(AStepsPerSec: DWORD): Double;
begin
  Result := AStepsPerSec * FUnitsPerStep * 60;
end;

function TGCodeParam.StepsPerSecondToVelocity(AStepsPerSec: DWORD;
                                              AUnitsPerStep: Double): Double;
begin
  Result := AStepsPerSec * AUnitsPerStep * 60;
end;

function TGCodeParam.VelocityToStepsPerSecond(AVelocity : Double): DWORD;
begin
  Result := Round(AVelocity / (FUnitsPerStep * 60));
end;

function TGCodeParam.VelocityToStepsPerSecond(AVelocity,
                                              AUnitsPerStep: Double): DWORD;
begin
  Result := Round(AVelocity / (AUnitsPerStep * 60));
end;

function TGCodeParam.StepsPerSecSecToAccel(AStepsPerSecSec: DWORD): Double;
begin
  Result := AStepsPerSecSec * FUnitsPerStep * 60 * 60;
end;

function TGCodeParam.StepsPerSecSecToAccel(AStepsPerSecSec: DWORD;
                                           AUnitsPerStep: Double): Double;
begin
  Result := AStepsPerSecSec * AUnitsPerStep * 60 * 60;
end;

function TGCodeParam.AccelToStepsPerSecSec(AAccel : Double): DWORD;
begin
  Result := Round(AAccel / (FUnitsPerStep * 60 * 60));
end;

function TGCodeParam.AccelToStepsPerSecSec(AAccel, AUnitsPerStep: Double): DWORD;
begin
  Result := Round(AAccel / (AUnitsPerStep * 60 * 60));
end;

function TGCodeParam.DwellToSG4Time(fTime: double): DWORD;
begin
  if FDwellUnit then //seconds
  begin
    if ftime > 20 then fTime := 20;
    Result := round(fTime * 1000000);
  end else
  begin
    //milliseconds
    if fTime > 20000 then fTime := 20000;
    Result := round(fTime * 1000);
  end;
end;

function TGCodeParam.ReadLengthUnit: Boolean;
begin
  Result := FLengthUnit;
end;

function TGCodeParam.ReadUnitsPerStep: Double;
begin
  Result := FUnitsPerStep;
end;

procedure TGCodeParam.SetLengthUnit(const Value: Boolean);
begin
  if FLengthUnit <> Value then
  begin
    FLengthUnit := Value;
    if FLengthUnit then
      FUnitsPerStep := MillimeterToInch(FUnitsPerStep)
    else FUnitsPerStep := InchToMillimeter(FUnitsPerStep);
  end;
end;

procedure TGCodeParam.SetUnitsPerStep(const Value: Double);
begin
  FUnitsPerStep := Value;
end;

function TGCodeParam.ReadDwellUnit: Boolean;
begin
  Result := FDwellUnit;
end;

procedure TGCodeParam.SetDwellUnit(const Value: Boolean);
begin
  FDwellUnit := Value;
end;

function TGCodeParam.Read_X: Double;
begin
  Result := FX;
end;

function TGCodeParam.Read_XSteps: Integer;
begin
  Result := LengthToSteps(FX);
end;

function TGCodeParam.Read_XChanged: Boolean;
begin
  Result := FXChanged;
end;

procedure TGCodeParam.Set_X(const Value: Double);
begin
  FX := Value;
  FXChanged := true;
end;

function TGCodeParam.Read_Y: Double;
begin
  Result := FY;
end;

function TGCodeParam.Read_YSteps: Integer;
begin
  Result := LengthToSteps(FY);
end;

function TGCodeParam.Read_YChanged: Boolean;
begin
  Result := FYChanged;
end;

procedure TGCodeParam.Set_Y(const Value: Double);
begin
  FY := Value;
  FYChanged := true;
end;

function TGCodeParam.Read_Z: Double;
begin
  Result := FZ;
end;

function TGCodeParam.Read_ZSteps: Integer;
begin
  Result := LengthToSteps(FZ);
end;

function TGCodeParam.Read_ZChanged: Boolean;
begin
  Result := FZChanged;
end;

procedure TGCodeParam.Set_Z(const Value: Double);
begin
  FZ := Value;
  FZChanged := true;
end;

function TGCodeParam.Read_I: Double;
begin
  Result := FI;
end;

function TGCodeParam.Read_ISteps: Integer;
begin
  Result := LengthToSteps(FI);
end;

function TGCodeParam.Read_IChanged: Boolean;
begin
  Result := FIChanged;
end;

procedure TGCodeParam.Set_I(const Value: Double);
begin
  FI := Value;
  FIChanged := true;
end;

function TGCodeParam.Read_J: Double;
begin
  Result := FJ;
end;

function TGCodeParam.Read_JSteps: Integer;
begin
  Result := LengthToSteps(FJ);
end;

function TGCodeParam.Read_JChanged: Boolean;
begin
  Result := FJChanged;
end;

procedure TGCodeParam.Set_J(const Value: Double);
begin
  FJ := Value;
  FJChanged := true;
end;

function TGCodeParam.Read_K: Double;
begin
  Result := FK;
end;

function TGCodeParam.Read_KSteps: Integer;
begin
  Result := LengthToSteps(FK);
end;

function TGCodeParam.Read_KChanged: Boolean;
begin
  Result := FKChanged;
end;

procedure TGCodeParam.Set_K(const Value: Double);
begin
  FK := Value;
  FKChanged := true;
end;

function TGCodeParam.Read_F: Double;
begin
  Result := FF;
end;

function TGCodeParam.Read_FChanged: Boolean;
begin
  Result := FFChanged;
end;

procedure TGCodeParam.Set_F(const Value: Double);
begin
  FF := Value;
  FFChanged := true;
end;

function TGCodeParam.Read_D: Double;
begin
  Result := FD;
end;

function TGCodeParam.Read_DChanged: Boolean;
begin
  Result := FDChanged;
end;

procedure TGCodeParam.Set_D(const Value: Double);
begin
  FD := Value;
  FDChanged := true;
end;

function TGCodeParam.Read_H: Double;
begin
  Result := FH;
end;

function TGCodeParam.Read_HChanged: Boolean;
begin
  Result := FHChanged;
end;

procedure TGCodeParam.Set_H(const Value: Double);
begin
  FH := Value;
  FHChanged := true;
end;

function TGCodeParam.Read_L: Double;
begin
  Result := FL;
end;

function TGCodeParam.Read_LChanged: Boolean;
begin
  Result := FLChanged;
end;

procedure TGCodeParam.Set_L(const Value: Double);
begin
  FL := Value;
  FLChanged := true;
end;

function TGCodeParam.Read_N: Double;
begin
  Result := FN;
end;

function TGCodeParam.Read_NChanged: Boolean;
begin
  Result := FNChanged;
end;

procedure TGCodeParam.Set_N(const Value: Double);
begin
  FN := Value;
  FNChanged := true;
end;

function TGCodeParam.Read_P: Double;
begin
  Result := FP;
end;

function TGCodeParam.Read_PChanged: Boolean;
begin
  Result := FPChanged;
end;

procedure TGCodeParam.Set_P(const Value: Double);
begin
  FP := Value;
  FPChanged := true;
end;

function TGCodeParam.Read_Q: Double;
begin
  Result := FQ;
end;

function TGCodeParam.Read_QChanged: Boolean;
begin
  Result := FQChanged;
end;

procedure TGCodeParam.Set_Q(const Value: Double);
begin
  FQ := Value;
  FQChanged := true;
end;

function TGCodeParam.Read_R: Double;
begin
  Result := FR;
end;

function TGCodeParam.Read_RChanged: Boolean;
begin
  Result := FRChanged;
end;

procedure TGCodeParam.Set_R(const Value: Double);
begin
  FR := Value;
  FRChanged := true;
end;

function TGCodeParam.Read_S: Double;
begin
  Result := FS;
end;

function TGCodeParam.Read_SChanged: Boolean;
begin
  Result := FSChanged;
end;

procedure TGCodeParam.Set_S(const Value: Double);
begin
  FS := Value;
  FSChanged := true;
end;

function TGCodeParam.Read_T: Double;
begin
  Result := FT;
end;

function TGCodeParam.Read_TChanged: Boolean;
begin
  Result := FTChanged;
end;

procedure TGCodeParam.Set_T(const Value: Double);
begin
  FT := Value;
  FTChanged := true;
end;

function TGCodeParam.Read_O: Double;
begin
  Result := FO;
end;

function TGCodeParam.Read_OChanged: Boolean;
begin
  Result := FOChanged;
end;

procedure TGCodeParam.Set_O(const Value: Double);
begin
  FO := Value;
  FOChanged := true;
end;

procedure TGCodeParam.ResetFlags;
begin
  FXChanged := false;
  FYChanged := false;
  FZChanged := false;
  FIChanged := false;
  FJChanged := false;
  FKChanged := false;
  FFChanged := false;
  FDChanged := false;
  FHChanged := false;
  FLChanged := false;
  FNChanged := false;
  FOChanged := false;
  FPChanged := false;
  FQChanged := false;
  FRChanged := false;
  FSChanged := false;
  FTChanged := false;
  FActiveMotionChanged := false;
end;

function TGCodeParam.ReadActiveMotion: Integer;
begin
  Result := FActiveMotion;
end;

procedure TGCodeParam.SetActiveMotion(const Value: Integer);
begin
  FActiveMotion := Value;
  FActiveMotionChanged := true;
end;

function TGCodeParam.ReadActiveMotionChanged: Boolean;
begin
  Result := FActiveMotionChanged;
end;

function TGCodeParam.ReadActivePlane: Integer;
begin
  Result := FActivePlane;
end;

procedure TGCodeParam.SetActivePlane(const Value: Integer);
begin
  FActivePlane := Value;
end;

function TGCodeParam.ReadVmaxSteps: DWORD;
begin
  Result := FVmax;
end;

function TGCodeParam.ReadVmaxUnits: double;
begin
  Result := StepsPerSecondToVelocity(FVmax);
end;

procedure TGCodeParam.SetVmaxSteps(const Value: DWORD);
begin
  FVmax := Value;
  if FVmax > FVmaxUpperLimit then FVmax := FVmaxUpperLimit;
  if FVmax < FVmaxLowerLimit then FVmax := FVmaxLowerLimit;
  SetVminPercentOfVmax(FVminPercentOfVmax);
end;

procedure TGCodeParam.SetVmaxUnits(const Value: double);
begin
  SetVmaxSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadVminSteps: DWORD;
begin
  Result := FVmin;
end;

function TGCodeParam.ReadVminUnits: Double;
begin
  Result := StepsPerSecondToVelocity(FVmin);
end;

procedure TGCodeParam.SetVminSteps(const Value: DWORD);
begin
  FVmin := Value;
  FVminPercentOfVmax := 0;
  SetVminPercentOfVmax(FVminPercentOfVmax);
end;

procedure TGCodeParam.SetVminUnits(const Value: Double);
begin
  SetVminSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadAccelSteps: DWORD;
begin
  Result := FAccel;
end;

function TGCodeParam.ReadAccelUnits: Double;
begin
  Result := StepsPerSecSecToAccel(FAccel);
end;

procedure TGCodeParam.SetAccelSteps(const Value: DWORD);
begin
  FAccel := Value;
  if FAccel > 100000 then FAccel := 100000;
  if FAccel = 0 then FAccel := 1;
  FAccelDecelTime := 0;
end;

procedure TGCodeParam.SetAccelUnits(const Value: Double);
begin
  SetAccelSteps(AccelToStepsPerSecSec(Value));
end;

function TGCodeParam.ReadDecelSteps: DWORD;
begin
  Result := FDecel;
end;

function TGCodeParam.ReadDecelUnits: Double;
begin
  Result := StepsPerSecSecToAccel(FDecel);
end;

procedure TGCodeParam.SetDecelSteps(const Value: DWORD);
begin
  FDecel := Value;
  if FDecel > 100000 then FDecel := 100000;
  if FDecel = 0 then FDecel := 1;
  FAccelDecelTime := 0;
end;

procedure TGCodeParam.SetDecelUnits(const Value: Double);
begin
  SetDecelSteps(AccelToStepsPerSecSec(Value));
end;

function TGCodeParam.ReadVminPercentOfVmax: Byte;
begin
  Result := FVminPercentofVmax;
end;

procedure TGCodeParam.SetVminPercentOfVmax(const Value: Byte);
begin
  FVminPercentOfVmax := Value;
  if FVminPercentOfVmax > 100 then FVminPercentOfVmax := 100; //100%

  if FVminPercentOfVmax > 0 then FVmin := Round(FVmax * FVminPercentOfVmax / 100)
  else if FVmin > FVmax then FVmin := FVmax;
  
  if FVmin > FVminUpperLimit then FVmin := FVminUpperLimit;
  if FVmin < FVminLowerLimit then FVmin := FVminLowerLimit;

  SetAccelDecelTime(FAccelDecelTime);
end;

function TGCodeParam.ReadIsVminPercentOfVmax: Boolean;
begin
  Result := (FVminPercentOfVmax > 0);
end;

function TGCodeParam.ReadAccelDecelTime: DWORD;
begin
  Result := FAccelDecelTime;
end;

procedure TGCodeParam.SetAccelDecelTime(const Value: DWORD);
var time_in_seconds : double;
begin
  FAccelDecelTime := Value; // time in milliseconds
  if FAccelDecelTime > 0 then
  begin     //FVmax & FVmin in [steps/second]
    time_in_seconds := FAccelDecelTime / 1000;
    FAccel := Round((FVmax - FVmin) / time_in_seconds);

    if FAccel > 100000 then FAccel := 100000;
    if FAccel = 0 then FAccel := 1;

    FDecel := FAccel;
    if FDecel > 100000 then FDecel := 100000;
    if FDecel = 0 then FDecel := 1;
  end;
end;

function TGCodeParam.ReadVmaxUpperLimitSteps: DWORD;
begin
  Result := FVmaxUpperLimit;
end;

procedure TGCodeParam.SetVmaxUpperLimitSteps(const Value: DWORD);
begin
  FVmaxUpperLimit := Value;

  If (FVmaxUpperLimit > 100000) OR
     (FVmaxUpperLimit = 0) then FVmaxUpperLimit := 100000;

  if FVmax > FVmaxUpperLimit then FVmax := FVmaxUpperLimit;
  if FVmaxLowerLimit > FVmaxUpperLimit then FVmaxLowerLimit := FVmaxUpperLimit;
  if FVminUpperLimit > FVmaxUpperLimit then FVminUpperLimit := FVmaxUpperLimit;
  if FVminLowerLimit > FVminUpperLimit then FVminLowerLimit := FVminUpperLimit;

  SetVminPercentOfVmax(FVminPercentOfVmax);
end;

function TGCodeParam.ReadVmaxUpperLimitUnits: Double;
begin
  Result := StepsPerSecondToVelocity(FVmaxUpperLimit);
end;

procedure TGCodeParam.SetVmaxUpperLimitUnits(const Value: Double);
begin
  SetVmaxUpperLimitSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadVminUpperLimitSteps: DWORD;
begin
  Result := FVminUpperLimit;
end;

procedure TGCodeParam.SetVminUpperLimitSteps(const Value: DWORD);
begin
  FVminUpperLimit := Value;
  if FVminUpperLimit = 0 then FVminUpperLimit := FVmaxUpperLimit;
  SetVmaxUpperLimitSteps(FVmaxUpperLimit);
end;

function TGCodeParam.ReadVminUpperLimitUnits: Double;
begin
  Result := StepsPerSecondToVelocity(FVminUpperLimit);
end;

procedure TGCodeParam.SetVminUpperLimitUnits(const Value: Double);
begin
  SetVminUpperLimitSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadVmaxLowerLimitSteps: DWORD;
begin
  Result := FVmaxLowerLimit;
end;

procedure TGCodeParam.SetVmaxLowerLimitSteps(const Value: DWORD);
begin
  FVmaxLowerLimit := Value;
  if FVmaxLowerLimit = 0 then FVmaxLowerLimit := 1;
  if FVmax < FVmaxLowerLimit then FVmax := FVmaxLowerLimit;
  SetVmaxUpperLimitSteps(FVmaxUpperLimit);
end;

function TGCodeParam.ReadVmaxLowerLimitUnits: Double;
begin
  Result := StepsPerSecondToVelocity(FVmaxLowerLimit);
end;

procedure TGCodeParam.SetVmaxLowerLimitUnits(const Value: Double);
begin
  SetVmaxLowerLimitSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadVminLowerLimitSteps: DWORD;
begin
  Result := FVminLowerLimit;
end;

procedure TGCodeParam.SetVminLowerLimitSteps(const Value: DWORD);
begin
  FVminLowerLimit := Value;
  if FVminLowerLimit = 0 then FVminLowerLimit := 1;
  SetVmaxUpperLimitSteps(FVmaxUpperLimit);
end;

function TGCodeParam.ReadVminLowerLimitUnits: Double;
begin
  Result := StepsPerSecondToVelocity(FVminLowerLimit);
end;

procedure TGCodeParam.SetVminLowerLimitUnits(const Value: Double);
begin
  SetVminLowerLimitSteps(VelocityToStepsPerSecond(Value));
end;

function TGCodeParam.ReadActiveFixture: Byte;
begin
  Result := FActiveFixture;
end;

procedure TGCodeParam.WriteActiveFixture(const Value: Byte);
begin
  FActiveFixture := Value;
  if FActiveFixture > 9 then FActiveFixture := 9;
end;

function TGCodeParam.ReadFixtureXOffSet: Double;
begin
  Result := FFixtureOffSets[FActiveFixture].X;
end;

procedure TGCodeParam.WriteFixtureXOffSet(const Value: Double);
begin
  FFixtureOffSets[FActiveFixture].X := Value;
end;

function TGCodeParam.ReadFixtureXOffSetSteps: Integer;
begin
  Result := LengthToSteps(FFixtureOffSets[FActiveFixture].X);
end;

procedure TGCodeParam.WriteFixtureXOffSetSteps(const Value: Integer);
begin
  FFixtureOffSets[FActiveFixture].X := StepsToLength(Value);
end;

function TGCodeParam.ReadFixtureYOffSet: Double;
begin
  Result := FFixtureOffSets[FActiveFixture].Y;
end;

procedure TGCodeParam.WriteFixtureYOffSet(const Value: Double);
begin
  FFixtureOffSets[FActiveFixture].Y := Value;
end;

function TGCodeParam.ReadFixtureYOffSetSteps: Integer;
begin
  Result := LengthToSteps(FFixtureOffSets[FActiveFixture].Y);
end;

procedure TGCodeParam.WriteFixtureYOffSetSteps(const Value: Integer);
begin
  FFixtureOffSets[FActiveFixture].Y := StepsToLength(Value);
end;

function TGCodeParam.ReadFixtureZOffSet: Double;
begin
  Result := FFixtureOffSets[FActiveFixture].Z;
end;

procedure TGCodeParam.WriteFixtureZOffSet(const Value: Double);
begin
  FFixtureOffSets[FActiveFixture].Z := Value;
end;

function TGCodeParam.ReadFixtureZOffSetSteps: Integer;
begin
  Result := LengthToSteps(FFixtureOffSets[FActiveFixture].Z);
end;

procedure TGCodeParam.WriteFixtureZOffSetSteps(const Value: Integer);
begin
  FFixtureOffSets[FActiveFixture].Z := StepsToLength(Value);
end;

function TGCodeParam.ReadDistanceMode: Integer;
begin
  Result := FDistanceMode;
end;

procedure TGCodeParam.SetDistanceMode(const Value: Integer);
begin
  FDistanceMode := Value;
end;

function TGCodeParam.ReadArcDistanceMode: Integer;
begin
  Result := FArcDistanceMode;
end;

procedure TGCodeParam.SetArcDistanceMode(const Value: Integer);
begin
  FArcDistanceMode := Value;
end;

end.
