unit uSG4Converter;

interface
uses Windows, Sysutils, Classes, udef, UnitErrorCode, uGCodeAnatomizer,
     uGCodeParameter;

type    TGCodeConverter = Class
        private
        protected
        public
          GCodeLine : TGCodeLine;
          SG4Cmds : TStringList;
          Params : TGCodeParam;

          constructor Create;
          destructor Destroy; override;


          function InchToMillimeter(AInch : Double) : Double;
          function MillimeterToInch(AMillimeter : Double) : Double;
          function LengthToSteps(ALength : Double) : Integer; Overload;
          function LengthToSteps(ALength, AUnitsPerStep : Double) : Integer; Overload;
          function StepsToLength(ASteps : Integer) : Double; Overload;
          function StepsToLength(ASteps : Integer; AUnitsPerStep : Double) : Double; Overload;
          function VelocityToStepsPerSecond(AVelocity : Double) : DWORD; Overload;
          function VelocityToStepsPerSecond(AVelocity, AUnitsPerStep : Double) : DWORD; Overload;
          function StepsPerSecondToVelocity(AStepsPerSec : DWORD) : Double; Overload;
          function StepsPerSecondToVelocity(AStepsPerSec : DWORD; AUnitsPerStep : Double) : Double; Overload;
          function AccelToStepsPerSecSec(AAccel : Double) : DWORD; Overload;
          function AccelToStepsPerSecSec(AAccel, AUnitsPerStep : Double) : DWORD; Overload;
          function StepsPerSecSecToAccel(AStepsPerSecSec : DWORD) : Double; Overload;
          function StepsPerSecSecToAccel(AStepsPerSecSec : DWORD; AUnitsPerStep : Double) : Double; Overload;

          function SG4Convert(AGCodeLine : TGCodeLine;  ALineNum : DWORD;
                              ADelete : Boolean = false) : DWORD;
          function GetSG4Code(AGCode : String; ALineNum : DWORD) : DWORD;
        end;

implementation

{ TGCodeConverter }

constructor TGCodeConverter.Create;
begin
  inherited Create;
  GCodeLine := TGCodeLine.Create;
  SG4Cmds := TStringList.Create;
  Params := TGCodeParam.Create;
end;

destructor TGCodeConverter.Destroy;
begin
  GCodeLine.Free;
  SG4Cmds.Free;
  Params.Free;
  inherited Destroy;
end;

function TGCodeConverter.InchToMillimeter(AInch: Double): Double;
begin
  Result := Params.InchToMillimeter(AInch);
end;

function TGCodeConverter.MillimeterToInch(AMillimeter: Double): Double;
begin
  Result := Params.MillimeterToInch(AMillimeter);
end;

function TGCodeConverter.LengthToSteps(ALength: Double): Integer;
begin
  Result := Params.LengthToSteps(Alength);
end;

function TGCodeConverter.LengthToSteps(ALength,
                                       AUnitsPerStep: Double): Integer;
begin
  Result := Params.LengthToSteps(Alength,AUnitsPerStep);
end;

function TGCodeConverter.StepsToLength(ASteps: Integer): Double;
begin
  Result := Params.StepsToLength(ASteps);
end;

function TGCodeConverter.StepsToLength(ASteps: Integer;
                                       AUnitsPerStep: Double): Double;
begin
  Result := Params.StepsToLength(ASteps,AUnitsPerStep);
end;

function TGCodeConverter.AccelToStepsPerSecSec(AAccel: Double): DWORD;
begin
  Result := Params.AccelToStepsPerSecSec(AAccel);
end;

function TGCodeConverter.AccelToStepsPerSecSec(AAccel,
                                               AUnitsPerStep: Double): DWORD;
begin
  Result := Params.AccelToStepsPerSecSec(AAccel,AUnitsPerStep);
end;

function TGCodeConverter.StepsPerSecSecToAccel(AStepsPerSecSec: DWORD): Double;
begin
  Result := Params.StepsPerSecSecToAccel(AStepsPerSecSec);
end;

function TGCodeConverter.StepsPerSecSecToAccel(AStepsPerSecSec: DWORD;
                                               AUnitsPerStep: Double): Double;
begin
  Result := Params.StepsPerSecSecToAccel(AStepsPerSecSec,AUnitsPerStep);
end;

function TGCodeConverter.StepsPerSecondToVelocity(AStepsPerSec: DWORD): Double;
begin
  Result := Params.StepsPerSecondToVelocity(AStepsPerSec);
end;

function TGCodeConverter.StepsPerSecondToVelocity(AStepsPerSec: DWORD;
                                                  AUnitsPerStep: Double): Double;
begin
  Result := Params.StepsPerSecondToVelocity(AStepsPerSec,AUnitsPerStep);
end;

function TGCodeConverter.VelocityToStepsPerSecond(AVelocity: Double): DWORD;
begin
  Result := Params.VelocityToStepsPerSecond(AVelocity);
end;

function TGCodeConverter.VelocityToStepsPerSecond(AVelocity,
                                                  AUnitsPerStep: Double): DWORD;
begin
  Result := Params.VelocityToStepsPerSecond(AVelocity,AUnitsPerStep);
end;

function TGCodeConverter.SG4Convert(AGCodeLine: TGCodeLine; ALineNum : DWORD;
                                    ADelete: Boolean): DWORD;
var loop : integer;
    fValue   : double;
    Idx, Old_Idx : Integer;
    scmd, snum : string;
begin

  snum := IntToStr(ALineNum);
  SG4Cmds.Clear;
  Params.ResetFlags;
  
  if AGCodeLine = nil then
  begin
    Result := ERR_NIL_GCODELINE_OBJECT;
    exit;
  end;
  if not AGCodeLine.anatomized then
  begin
    Result := ERR_GCODELINE_OBJECT_EMPTY;
    exit;
  end;

  Old_Idx := -1;
  for loop := 0 to length(cGroup_G6)-1 do
  begin
    Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,cGroup_G6[loop]);
    if (Idx >= 0) and (Old_Idx >= 0) then
    begin
      Result := ERR_MULTI_GCODE;
      exit;
    end;
    if Idx >= 0 then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      Params.LengthUnit := (cGroup_G6[loop] = G_20);
      Old_Idx := Idx;
    end;
  end;

  for loop := 0 to length(cParams)-1 do
  begin
    case AGCodeLine.get_param_double(cParams[loop],fValue,ADelete) of
    -1 : ;
    0 : begin
          case cParams[loop] of
          'D': params._D := fvalue;
          'F': params._F := fvalue;
          'H': params._H := fvalue;
          'I': params._I := fvalue;
          'J': params._J := fvalue;
          'K': params._K := fvalue;
          'L': params._L := fvalue;
          'N': params._N := fvalue;
          'O': params._O := fvalue;
          'P': params._P := fvalue;
          'Q': params._Q := fvalue;
          'R': params._R := fvalue;
          'S': params._S := fvalue;
          'T': params._T := fvalue;
          'X': params._X := fvalue;
          'Y': params._Y := fvalue;
          'Z': params._Z := fvalue;
          end;
          AGCodeLine.ParamList.Delete(AGCodeLine.ParamList.IndexOf(cParams[loop]));
        end;
    1 : begin
          Result := ERR_BAD_NUMBER;
          exit;
        end;
    end;
  end;

  if Params._FChanged then
  begin
    Params.VmaxUnits := Params._F;
    SG4Cmds.AddObject('s5:'+IntToStr(Params.VmaxSteps),TObject(ALineNum));
    SG4Cmds.AddObject('s6:'+IntToStr(Params.VminSteps),TObject(ALineNum));
    SG4Cmds.AddObject('s7:'+IntToStr(Params.AccelSteps),TObject(ALineNum));
    SG4Cmds.AddObject('s8:'+IntToStr(Params.DecelSteps),TObject(ALineNum));
  end;

  if Params._SChanged then
  begin
    //not supported yet!
    //Result := ERR_GCODE_NOT_SUPPORTED;
    //Exit;
  end;

  if Params._TChanged then
  begin
    //not supported yet!
    //Result := ERR_GCODE_NOT_SUPPORTED;
    //Exit;
  end;


  Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,G_40);
  if Idx >= 0 then
  begin
    if Params._PChanged then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      SG4Cmds.AddObject('s40:'+IntToStr(Params.DwellToSG4Time(Params._P)),TObject(ALineNum));
    end else
    begin
      Result := ERR_MISSING_PARAM;
      Exit;
    end;
  end;

  Old_Idx := -1;
  for loop := 0 to length(cGroup_G2)-1 do
  begin
    Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,cGroup_G2[loop]);
    if (Idx >= 0) and (Old_Idx >= 0) then
    begin
      Result := ERR_MULTI_GCODE;
      exit;
    end;
    if Idx >= 0 then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      Params.ActivePlane := cGroup_G2[loop];
      Old_Idx := Idx;
    end;
  end;

  Old_Idx := -1;
  for loop := 0 to length(cGroup_G3)-1 do
  begin
    Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,cGroup_G3[loop]);
    if (Idx >= 0) and (Old_Idx >= 0) then
    begin
      Result := ERR_MULTI_GCODE;
      exit;
    end;
    if Idx >= 0 then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      Params.DistanceMode := cGroup_G3[loop];
      Old_Idx := Idx;
    end;
  end;

  Old_Idx := -1;
  for loop := 0 to length(cGroup_G3_1)-1 do
  begin
    Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,cGroup_G3_1[loop]);
    if (Idx >= 0) and (Old_Idx >= 0) then
    begin
      Result := ERR_MULTI_GCODE;
      exit;
    end;
    if Idx >= 0 then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      Params.ArcDistanceMode := cGroup_G3_1[loop];
      Old_Idx := Idx;
    end;
  end;

  Old_Idx := -1;
  for loop := 0 to length(cGroup_G1)-1 do
  begin
    Idx := AGCodeLine.IndexOfInt(AGCodeLine.GCodeList,cGroup_G1[loop]);
    if (Idx >= 0) and (Old_Idx >= 0) then
    begin
      Result := ERR_MULTI_GCODE;
      exit;
    end;
    if Idx >= 0 then
    begin
      AGCodeLine.GCodeList.Delete(Idx);
      Params.ActiveMotion := cGroup_G1[loop];
      Old_Idx := Idx;
    end;
  end;

  if Params._XChanged or Params._YChanged or Params._ZChanged then
  begin
    case Params.ActiveMotion of
    G_0,
    G_1 : begin
            if Params.DistanceMode = G_90 then
            begin
              scmd := 's51:N'+snum;
              if Params._XChanged then scmd := scmd + 'X'+IntToStr(Params._XSteps + Params.XOffSetSteps);
              if Params._YChanged then scmd := scmd + 'Y'+IntToStr(Params._YSteps + Params.YOffSetSteps);
              if Params._ZChanged then scmd := scmd + 'Z'+IntToStr(Params._ZSteps + Params.ZOffSetSteps);
            end else
            begin
              scmd := 's50:N'+snum;
              if Params._XChanged then scmd := scmd + 'X'+IntToStr(Params._XSteps);
              if Params._YChanged then scmd := scmd + 'Y'+IntToStr(Params._YSteps);
              if Params._ZChanged then scmd := scmd + 'Z'+IntToStr(Params._ZSteps);
            end;
          end;
    G_2,
    G_3 : begin
            if Params.ActiveMotion = G_2 then //CW
            begin
              if Params.DistanceMode = G_90 then //absolute
              begin
                if Params.ArcDistanceMode = G_90_1 then
                   scmd := 's54:N'+snum // CW; absolute endpoint; absolute center
                else scmd := 's52:N'+snum; //CW; absolute endpoint; incremental center
              end else
              begin  //incremental
                if Params.ArcDistanceMode = G_90_1 then
                  scmd := 's58:N'+snum // CW; incremental endpoint; absolute center
                else scmd := 's56:N'+snum; // CW; incremental endpoint; incremental center
              end;
            end else
            begin    // CCW
              if Params.DistanceMode = G_90 then //absolute
              begin
                if Params.ArcDistanceMode = G_90_1 then
                   scmd := 's55:N'+snum // CCW; absolute endpoint; absolute center
                else scmd := 's53:N'+snum; //CCW; absolute endpoint; incremental center
              end else
              begin  //incremental
                if Params.ArcDistanceMode = G_90_1 then
                  scmd := 's59:N'+snum // CCW; incremental endpoint; absolute center
                else scmd := 's57:N'+snum; // CCW; incremental endpoint; incremental center
              end;
            end;
            Case Params.ActivePlane of
            G_17: begin //XY
                    if (not (Params._XChanged and Params._YChanged)) OR
                       ((not Params._IChanged) or (not Params._JChanged)) then
                    begin
                      Result := ERR_MISSING_PARAM;
                      exit;
                    end;
                    if Params.DistanceMode = G_90 then
                      scmd := scmd + 'X'+IntToStr(Params._XSteps + Params.XOffSetSteps)+
                                     'Y'+IntToStr(Params._YSteps + Params.YOffSetSteps)
                    else
                      scmd := scmd + 'X'+IntToStr(Params._XSteps)+
                                     'Y'+IntToStr(Params._YSteps);

                    if Params.ArcDistanceMode = G_91_1 then
                    begin
                      if Params._IChanged then scmd := scmd + 'I'+IntToStr(Params._ISteps);
                      if Params._JChanged then scmd := scmd + 'J'+IntToStr(Params._JSteps);
                    end else
                    begin
                      if Params._IChanged then scmd := scmd + 'I'+IntToStr(Params._ISteps + Params.XOffSetSteps);
                      if Params._JChanged then scmd := scmd + 'J'+IntToStr(Params._JSteps + Params.YOffSetSteps);
                    end;
                  end;
            G_18: begin //XZ
                    if (not (Params._XChanged and Params._ZChanged)) OR
                       ((not Params._IChanged) or (not Params._KChanged)) then
                    begin
                      Result := ERR_MISSING_PARAM;
                      exit;
                    end;
                    if Params.DistanceMode = G_90 then
                      scmd := scmd + 'X'+IntToStr(Params._XSteps + Params.XOffSetSteps)+
                                     'Z'+IntToStr(Params._ZSteps + Params.ZOffSetSteps)
                    else
                      scmd := scmd + 'X'+IntToStr(Params._XSteps)+
                                     'Z'+IntToStr(Params._ZSteps);
                    if Params.ArcDistanceMode = G_91_1 then
                    begin
                      if Params._IChanged then scmd := scmd + 'I'+IntToStr(Params._ISteps);
                      if Params._KChanged then scmd := scmd + 'J'+IntToStr(Params._KSteps);
                    end else
                    begin
                      if Params._IChanged then scmd := scmd + 'I'+IntToStr(Params._ISteps + Params.XOffSetSteps);
                      if Params._KChanged then scmd := scmd + 'J'+IntToStr(Params._KSteps + Params.ZOffSetSteps);
                    end;
                  end;
            G_19: begin //YZ
                    if (not (Params._YChanged and Params._ZChanged)) OR
                       ((not Params._JChanged) or (not Params._KChanged)) then
                    begin
                      Result := ERR_MISSING_PARAM;
                      exit;
                    end;
                    if Params.DistanceMode = G_90 then
                      scmd := scmd + 'Y'+IntToStr(Params._YSteps + Params.YOffSetSteps)+
                                     'Z'+IntToStr(Params._ZSteps + Params.ZOffSetSteps)
                    else
                      scmd := scmd + 'Y'+IntToStr(Params._YSteps)+
                                     'Z'+IntToStr(Params._ZSteps);
                    if Params.ArcDistanceMode = G_91_1 then
                    begin
                      if Params._JChanged then scmd := scmd + 'I'+IntToStr(Params._JSteps);
                      if Params._KChanged then scmd := scmd + 'J'+IntToStr(Params._KSteps);
                    end else
                    begin
                      if Params._JChanged then scmd := scmd + 'I'+IntToStr(Params._JSteps + Params.YOffSetSteps);
                      if Params._KChanged then scmd := scmd + 'J'+IntToStr(Params._KSteps + Params.ZOffSetSteps);
                    end;
                  end;
            end;
          end;
    end;
    SG4Cmds.AddObject(scmd,TObject(ALineNum));
  end;

  Result := RET_OK;
end;

function TGCodeConverter.GetSG4Code(AGCode: String; ALineNum: DWORD): DWORD;
var DataAvailable : Boolean;
begin
  Result := GCodeLine.anatomize(AGCode,DataAvailable);
  if (Result = RET_OK) and (DataAvailable) then
    Result := SG4Convert(GCodeLine,ALineNum,true);
end;

end.
